Attribute VB_Name = "LEF_Utilties_2014_11_03"
Option Explicit
Option Compare Text ' Non-case sensitive

'   * Copyright  2009-2014 by John P. Desautels (John@SmartBizNiz.com)
'   * Permission is granted to any individual/institution/corporate entity to use, copy, redistribute
'   * or modify this software for any purpose whatsoever, subject to one condition: namely, that this
'   * copyright block, in its entirety, not be removed from the source code.
'   *
'   * Source code, samples and documentation can be found at http://www.SmartBizNiz.com/downloads.html
'   *
'   * Special thanks to:
'   *   Ted Rau who introduced me to Launch Excel and inspired me to develop and share this framework
'   *   Guy Brown who helped me as I developed intricate YSL reports, which are the genesis of this framework
'   *   Deborah Brown, my first Yardi report writing instructor, who explained the history behind Launch Excel
'   *   Geoffrey Gelb who provided interesting projects which have allowed me to enhance the framework in support
'   *       of their requirements
'   *   All the Yardi VBA developers who have come before me, paving the way for this framework

Public Function WorksheetExists(strWorkbook As String, strWorksheet As String, Optional boolLogError As Boolean) As Boolean
' Test to see if a worksheet exists
' 2010-04-28 JPD Added boolLogError

    On Error GoTo ErrorHandler
    
    If UCase(Application.Workbooks(strWorkbook).Worksheets(strWorksheet).Name) = UCase(strWorksheet) Then
        WorksheetExists = True
    End If

ExitProcedure:
    
    Exit Function
    
ErrorHandler:
    ' If there is any error then this is not a valid worksheet
    WorksheetExists = False
    If boolLogError Then
        Call LogError("WorksheetExists", strWorkbook & " does not have a worksheet named " & strWorksheet)
    End If
     
    Resume ExitProcedure

End Function

Public Function FirstMatch(strSearchText As String, strWorkbook As String, strWorksheet As String, strRange As String) As Long
' Returns the relative position of the first match. -1 means there was no match
' Advantage over using the Match function directly is error handling
    On Error GoTo ErrorHandler

    FirstMatch = Application.WorksheetFunction.Match(strSearchText, Application.Workbooks(strWorkbook).Worksheets(strWorksheet).Range(strRange), 0)

ExitProcedure:
    
    Exit Function

ErrorHandler:
    ' If there is any error then strSearchText does not exist
    FirstMatch = -1
     
    Resume ExitProcedure
   
End Function

Public Sub LogError(strSource As String, strError As String)

    Dim intCurrentRow As Long
    
    On Error GoTo ErrorHandler

    
    ' Add an error page if one doesn't exist
    With ThisWorkbook
        If Not WorksheetExists(ThisWorkbook.Name, "Errors") Then
            ' Add Errors worksheet to myXLS if it doesn't exist
            .Worksheets.Add(after:=Worksheets(.Worksheets.Count)).Name = "Errors"
            .Worksheets("Errors").Cells(1, 1).Value = "Source"
            .Worksheets("Errors").Cells(1, 2).Value = "Error"
            .Worksheets("Errors").Cells(1, 3).Value = "Last Good Statement"
        End If
    
        intCurrentRow = ThisWorkbook.Worksheets("Errors").UsedRange.Rows.Count + 1
        
        ' Add the error to the bottom of the page
        .Worksheets("Errors").Cells(intCurrentRow, 1).Value = strSource
        .Worksheets("Errors").Cells(intCurrentRow, 2).Value = strError
        .Worksheets("Errors").Cells(intCurrentRow, 3).Value = strLastGoodStatement
    
        ' Format the column width
        .Worksheets("Errors").Cells(intCurrentRow, 1).EntireColumn.AutoFit
        .Worksheets("Errors").Cells(intCurrentRow, 2).EntireColumn.AutoFit
        .Worksheets("Errors").Cells(intCurrentRow, 3).EntireColumn.AutoFit

        ' Format the row height
        .Worksheets("Errors").Cells(intCurrentRow, 1).EntireRow.AutoFit
    End With
    
ExitProcedure:
    
    Exit Sub

ErrorHandler:
    ' If there is any error logging the errors then we are done.
     
    Resume ExitProcedure

End Sub

Function NamedRangeExists(Wks As Worksheet, strRangeName As String, boolLogError As Boolean, Optional boolGlobal As Boolean) As Boolean
    
On Error GoTo ErrorHandler
    
    Select Case boolGlobal
        Case True
            If Names(strRangeName).Index > 0 Then
                NamedRangeExists = True
            End If
        
        Case False
            If Wks.Names(strRangeName).Index > 0 Then
                NamedRangeExists = True
            End If
        
    End Select
    
    
ExitProcedure:
    Exit Function
    
ErrorHandler:
    ' If there are any errors then the range name doesn't exist
    If boolLogError Then
        Call LogError("NamedRangeExists", Wks.Name & " does not have the named range " & strRangeName)
    End If
    
    Resume ExitProcedure

End Function

Function ShapeExists(Wks As Worksheet, strShapeName As String, boolLogError As Boolean) As Boolean
    
On Error GoTo ErrorHandler
    
            If Wks.Shapes(strShapeName).Name = strShapeName Then
                ShapeExists = True
            End If
        
ExitProcedure:
    Exit Function
    
ErrorHandler:
    ' If there are any errors then the range name doesn't exist
    If boolLogError Then
        Call LogError("ShapeExists", Wks.Name & " does not have the shape named " & strShapeName)
    End If
    
    Resume ExitProcedure

End Function

Function KillFile(strFullFileName As String, boolLogError As Boolean) As Boolean
    
On Error GoTo ErrorHandler
    
    ' Try to kill the file
    Kill strFullFileName
    'If we made it this far, the file was deleted
    KillFile = True
        
ExitProcedure:
    Exit Function
    
ErrorHandler:
    ' If there are any errors then the range name doesn't exist
    If boolLogError Then
        Call LogError("KillFile", strFullFileName & " could not be deleted")
    End If
    
    Resume ExitProcedure

End Function

Function SaveWorkbook(strWorkbook As String, strFullFileName As String, boolLogError As Boolean, boolUpdateMyXLS As Boolean) As Boolean
    ' 2009-08-03 JPD    Added code to update myXLS
    
On Error GoTo ErrorHandler
    
    Dim strFileName As String
    
    ' Attempt to save the file
    Application.Workbooks(strWorkbook).SaveAs FileName:=strFullFileName, AddToMru:=True
    
    ' Get the file name without the path
    strFileName = Mid(strFullFileName, InStrRev(strFullFileName, "\") + 1)
    
    ' Flag as saved
    Application.Workbooks(strFileName).Saved = True
    
    ' Update myXLS with the new name
    If boolUpdateMyXLS Then myXLS = strFileName

    SaveWorkbook = True
    
ExitProcedure:
    Exit Function
    
ErrorHandler:
    ' If there are any errors then the file wasn't saved
    If boolLogError Then
        Call LogError("SaveWorkbook", strWorkbook & " could not be saved as " & strFileName)
    End If
    
    Resume ExitProcedure

End Function

Sub DeleteBadRefs(Optional strXLS As String)
    ' 2009-08-11 JPD    Created
    
    Dim nm As Name
  
    If strXLS = "" Then strXLS = ThisWorkbook.Name
    
    For Each nm In ActiveWorkbook.Names
        If InStr(1, nm.RefersTo, "#REF") > 0 Then
            'List the name before deleting
            Debug.Print nm.Name & ": deleted"
            nm.Delete
        End If
    Next nm
End Sub

Sub Reset_NamedRanges(strWorksheet As String)
    ' 2009-08-11 JPD    Created
    
    '                   This deletes the data from all visible locally scoped named ranges on a worksheet

    Dim nm As Name
    
    For Each nm In Sheets(strWorksheet).Names
        If nm.Visible And Right(nm.Name, 12) <> "Print_Titles" Then nm.RefersToRange.Value = ""
    Next nm
    
End Sub

Sub Reset_Data(strWorksheet As String, strRangeName As String, strMode As String)
    ' 2009-08-10 JPD    Created
    ' 2009-08-12 JPD    Added strMode
    '                    - "Clear" will clear all data and formatting, but does not delete the rows. UsedRange is NOT reset.
    '                    - "Values" will set the values to Empty, but does not clear the formatting or delete the rows. UsedRange is NOT reset.
    '                    - "Delete" will delete all the rows below the named range, remove some formatting from the first row (bold, italic, borders)
    '                       and set the values in the 1st row of the named range to Empty. UsedRange is reset.
    
    Dim intCurrentRow As Long
    
    ' Step 1 - Determine if the named range exists
    If NamedRangeExists(Sheets(strWorksheet), strRangeName, True, False) Then
        intCurrentRow = Range(Sheets(strWorksheet).Names(strRangeName).Value).Row
    Else
        Exit Sub
    End If

    
    Select Case strMode
        Case "Clear"
            If intCurrentRow <= Sheets(strWorksheet).UsedRange.Rows.Count Then
                ' Clear all rows, including the first row of the named range. This will get rid of any formatting but does NOT reset the UsedRange.
                Sheets(strWorksheet).Rows(intCurrentRow & ":" & Sheets(strWorksheet).UsedRange.Rows.Count + 1).Clear
            End If
        Case "Values"
            If intCurrentRow <= Sheets(strWorksheet).UsedRange.Rows.Count Then
                ' Set all values to Empty. Formatting is preserved. This does NOT reset the UsedRange.
                Sheets(strWorksheet).Rows(intCurrentRow & ":" & Sheets(strWorksheet).UsedRange.Rows.Count + 1).Value = Empty
            End If
        Case "Delete"
            If intCurrentRow < Sheets(strWorksheet).UsedRange.Rows.Count Then
                ' Delete all rows below the first row of the named range. This will get rid of any formatting as well as reset the UsedRange.
                Sheets(strWorksheet).Rows(intCurrentRow + 1 & ":" & Sheets(strWorksheet).UsedRange.Rows.Count + 1).Delete
            End If
            
            ' First row - Set value to Empty and remove formatting we may have added. Don't want to delete it or we will delete the named range as well
            With Sheets(strWorksheet).Rows(intCurrentRow)
                .Value = Empty
                .Borders(xlEdgeTop).LineStyle = xlNone
                .Borders(xlEdgeBottom).LineStyle = xlNone
                .Font.Bold = False
                .Font.Italic = False
                .Locked = True
                .Interior.ColorIndex = xlNone
            End With
    End Select
    
End Sub

Sub SelfDestruct()
    '2009-08-14 JPD Created to get the size of each worksheet
    
    Dim lngSizeBefore As Long
    Dim lngSizeAfter As Long
    Dim Wks As Worksheet
    Dim strWks As String
    
    If ThisWorkbook.Name <> "SelfDestruct.xls" Then
        MsgBox "This proceedure will only run if the workbook is named SelfDestruct.xls. Please save the file as SelfDestruct.xls and try again."
        Exit Sub
    End If
    
    With Application
        ' Temporarily disable certain Excel functionality
        .ScreenUpdating = False
        .DisplayAlerts = False
    End With
        
    strLastGoodStatement = "Worhsheet Size"
    
    lngSizeAfter = FileLen(ThisWorkbook.Name)
    
    Call LogError("Worksheet", "File Size After")
    
    Call LogError("Original Workbook", Str(lngSizeAfter))
    
    For Each Wks In ThisWorkbook.Worksheets
        If Wks.Name <> "Errors" Then
            lngSizeBefore = lngSizeAfter
            strWks = Wks.Name
            'Delete the sheet
            Wks.Delete
            
            'save the file
            ThisWorkbook.Save
            ' Get the new file size
            lngSizeAfter = FileLen(ThisWorkbook.Name)
            
            strLastGoodStatement = Str(lngSizeBefore - lngSizeAfter)
            
            Call LogError(strWks, Str(lngSizeAfter))
            
        End If
        
    Next Wks
    
    With Application
        ' Enable certain Excel functionality
        .ScreenUpdating = True
        .DisplayAlerts = True
    End With
    
End Sub

Function SetToNothing(ParamArray vObjects() As Variant) As Boolean
' 2009-08-14 JPD Created from http://www.vbaexpress.com/kb/getarticle.php?kb_id=793

    Dim i As Long
    
    On Error Resume Next
    
    SetToNothing = True
    
    For i = LBound(vObjects) To UBound(vObjects)
        Set vObjects(i) = Nothing
        If Err.Number <> 0 Then SetToNothing = False 'if not object/variant variable
    Next i

End Function

Sub RemoveExternalLinks(strWorkbook As String, Optional boolLogError As Boolean)
' 2009-11-05 JPD Created to remove links in Book1 that point back to template XLS
' 2009-11-11 JPD Changed from If InStr(Cell.FormulaLocal, ".xls]") > 0 to Do While If InStr(...
'                  This will catch multiple instances in the same formula
' 2009-12-04 JPD Changed from InStr(strFormula_Linked, "'") to InStr(strFormula_Linked, "'[")
' 2011-04-27 JPD Added On Error Resume Next, On Error GoTo 0, If Not rngFormula Is Nothing Then
'                  This is a better way to handle sheets with no formulas
' 2011-08-15 JPD Added InStr(strFormula_Linked, "'")

    Dim Wks As Worksheet
    Dim rngFormula As Range
    Dim Cell As Range
    Dim strFormula_Linked As String
    Dim strFormula_Unlinked As String
    Dim iErrorCount As Integer
    
    On Error GoTo ErrorHandler
    
    For Each Wks In Application.Workbooks(strWorkbook).Worksheets
            ' Narrow the search to cells with a formula
            ' This will return an error 1004 if the sheet has no formulas
            On Error Resume Next
            Set rngFormula = Wks.Cells.SpecialCells(xlFormulas)
            On Error GoTo 0
            
            If Not rngFormula Is Nothing Then
                For Each Cell In rngFormula
                    ' Inspect each cell formula cell to see if it is linked
                    Do While InStr(Cell.FormulaLocal, ".xls]") > 0
                        strFormula_Linked = Cell.Formula
                        ' Now check to see if there is a single quote followed by a left bracket
                        If InStr(strFormula_Linked, "'[") > 0 Then
                            strFormula_Unlinked = Left(strFormula_Linked, InStr(strFormula_Linked, "'[")) & _
                                Right(strFormula_Linked, Len(strFormula_Linked) - InStr(strFormula_Linked, ".xls]") - 4)
                        ' Next check to see if there is simply a single quote before ".xls]" 2011-08-15 JPD Added
                        ElseIf InStr(strFormula_Linked, "'") > 0 And InStr(strFormula_Linked, "'") < InStr(Cell.FormulaLocal, ".xls]") Then
                            strFormula_Unlinked = Left(strFormula_Linked, InStr(strFormula_Linked, "'") - 1) & _
                                Right(strFormula_Linked, Len(strFormula_Linked) - InStr(strFormula_Linked, ".xls]") - 4)
                        Else
                        ' No single quotes
                            strFormula_Unlinked = Left(strFormula_Linked, InStr(strFormula_Linked, "[") - 1) & _
                                Right(strFormula_Linked, Len(strFormula_Linked) - InStr(strFormula_Linked, ".xls]") - 4)
                        End If
                        
                        Cell.Formula = strFormula_Unlinked
                    Loop
                Next Cell
            End If
    Next Wks

ExitProcedure:
    Set rngFormula = Nothing
    Exit Sub
    
ErrorHandler:
    Select Case Err.Number
        Case 1004
            ' There are no formula cells in this worksheet
            iErrorCount = iErrorCount + 1
            ' Avoid getting caught in an infinite loop
            If iErrorCount > 100 Then
                If boolLogError Then
                    Call LogError("RemoveExternalLinks", strWorkbook & " had an error on " & Wks.Name)
                End If
                Resume ExitProcedure
            End If
            
            Resume Next
        Case Else
           ' If there are any errors then the range name doesn't exist
            If boolLogError Then
                Call LogError("RemoveExternalLinks", strWorkbook & " had an error on " & Wks.Name)
            End If
            
            Resume ExitProcedure
    End Select
    
End Sub

Sub Protect_Workbook(strXLS As String, strPassword As String)
'   2010-01-26 JPD  Created
'   2012-02-17 JPD  Modified to accomodate Excel 2000 for older Conductor servers

    Dim Wks As Worksheet
    
    For Each Wks In Workbooks(strXLS).Worksheets
        With Wks
            .EnableOutlining = True
            
            If Wks.Name <> "CSV" And Wks.Name <> "Errors" Then
                If Application.Version <= 9 Then    ' Excel 2000
                    .Protect Password:=strPassword, DrawingObjects:=True, Contents:=True, Scenarios:=True
                Else
                    .Protect Password:=strPassword, DrawingObjects:=True, Contents:=True, Scenarios:=True, _
                        AllowFormattingCells:=True, AllowFormattingColumns:=True, AllowFormattingRows:=True, _
                        AllowInsertingColumns:=True, AllowInsertingRows:=True, AllowDeletingColumns:=True, _
                        AllowDeletingRows:=True

                End If
                
            End If
            
            If Wks.Name = "Admin" Then
                .Visible = xlSheetVeryHidden
            End If
        
        End With
    Next Wks
    
End Sub

Sub Unprotect_Workbook(strXLS As String, strPassword As String)
'   2010-01-26 JPD  Created
    
    Dim Wks As Worksheet
    
    For Each Wks In Workbooks(strXLS).Worksheets
        With Wks
            .Unprotect Password:=strPassword
            If Wks.Name = "Admin" Then
                .Visible = True
            End If

        End With
    Next Wks
    
End Sub

Public Sub TestFormat()

    ' This procedure creates a copy of the active worksheet named TestFormat (Results)
    ' and runs the FormatCells function on the copy
    ' - This provides a way to test syntax of various FormatCells commands
    ' - #Titles_XXX is not supported, unless a properly formatted worksheet named named LEF_Titles exists
    ' - Existing worksheet named TestFormat (Results) will be deleted
    
    ' 2012-05-06 JPD Created
    ' 2013-10-06 JPD Added support for PageHeaderFooter
    
    On Error GoTo ErrorHandler
    
    Dim strXLS As String
    Dim strSource As String
    Dim strReport As String
    Dim strErrorMessage As String
    Dim strLastGoodStatement As String
    
    strXLS = Application.ActiveWorkbook.Name
    strSource = Application.ActiveSheet.Name
    strReport = strSource + " (Results)"
    
    ' Delete the Results tab, if it exists
    If WorksheetExists(strXLS, strReport) Then
        Application.DisplayAlerts = False
        Application.Workbooks(strXLS).Worksheets(strReport).Delete
        Application.DisplayAlerts = False
        strLastGoodStatement = "Delete " + strReport
    End If
             
    ' Copy strSource to strReport
    If WorksheetExists(strXLS, strSource, True) Then
        ' 2010-10-19 JPD Updated for Excel 2010 compatibility. Excel 2010 does not always name the copied sheet the same as Excel 2007 and earlier did.
        '                Therefore, we will copy it to the end, rename it and then move it.
        Sheets(strSource).Copy after:=Sheets(Sheets.Count)
        ' TODO: Verify that the report name is valid length and characters. Ideally, this would be done in the SQL script.
        Sheets(Sheets.Count).Name = strReport
        Sheets(strReport).Move after:=Sheets(strSource)
        strLastGoodStatement = "Copy " + strSource + " to " + strReport
    End If
    
    ' #Hyperlink_
    Call HyperlinkCreate(strXLS, strReport, "#Hyperlink_")
    strLastGoodStatement = "HyperlinkCreate #Hyperlink_(" + strXLS + "," + strReport + ")"
    
    ' #FC_
    Call FormatCells(strXLS, strReport, "#FC_", "Format")
    strLastGoodStatement = "FormatCells #FC_(" + strXLS + "," + strReport + ")"

    ' PageHeaderFooter
    Call PageHeaderFooter(strXLS, strReport, "")
    strLastGoodStatement = "PageHeaderFooter (" + strXLS + "," + strReport + ")"
    
    ' #FCAfter_
    Call FormatCells(strXLS, strReport, "#FCAfter_", "Format")
    strLastGoodStatement = "FormatCells #FCAfter_(" + strXLS + "," + strReport + ")"
    
    ' PageHeaderFooterAfter
    Call PageHeaderFooter(strXLS, strReport, "After")
    strLastGoodStatement = "PageHeaderFooterAfter (" + strXLS + "," + strReport + ")"
    
    ' #Picture_
    Call PictureInsert(strXLS, strReport, "#Picture_")
    strLastGoodStatement = "PictureInsert #Picture_(" + strXLS + "," + strReport + ")"
    
    ' #FCProtect_
    Call FormatCells(strXLS, strReport, "#FCProtect_", "Format")
    strLastGoodStatement = "FormatCells #FCProtect_(" + strXLS + "," + strReport + ")"
    
    ' PageHeaderFooterProtect
    Call PageHeaderFooter(strXLS, strReport, "Protect")
    strLastGoodStatement = "PageHeaderFooterProtect (" + strXLS + "," + strReport + ")"
    
    ' #FCFinal_
    Call FormatCells(strXLS, strReport, "#FCFinal_", "Format")
    strLastGoodStatement = "FormatCells #FCFinal_(" + strXLS + "," + strReport + ")"
    
    ' PageHeaderFooterFinal
    Call PageHeaderFooter(strXLS, strReport, "Final")
    strLastGoodStatement = "PageHeaderFooterFinal (" + strXLS + "," + strReport + ")"
    
    ' Select the results tab
    Sheets(strReport).Select
    
    ' Delete btnTestFormat from strReport
    Sheets(strReport).Shapes("btnTestFormat").Delete
    
ExitProcedure:
    Exit Sub
    
ErrorHandler:
    Select Case Err.Number
        Case Else
            strErrorMessage = Err.Number & ": " & Err.Description & vbCrLf & _
                "Last good statement: " & strLastGoodStatement
                
                MsgBox strErrorMessage, vbOKOnly, "TestFormat"
    End Select
    Resume ExitProcedure
    

End Sub


Function CreateUnicodeCSV(strCharset, strSourceFile, strDestinationFile, Optional boolLogError As Boolean) As Boolean
    ' This procedure creates a new CSV file (strDestinationFile), adding a a BOM (byte order mark) to the beginning of strSourceFile
    ' strCharset will generally be "UTF-8"
    
    ' 2013-01-11 JPD Created
    
    On Error GoTo ErrorHandler
    
    'Create a stream object
    Dim objStream As Object
    Set objStream = CreateObject("ADODB.Stream")
  
    With objStream
        .Charset = strCharset
        .Type = 2 'adTypeText
        .Open
        .LoadFromFile strSourceFile
        .SaveToFile strDestinationFile, 2 'adSaveCreateOverWrite
    End With
  
    CreateUnicodeCSV = True
    
ExitProcedure:
    objStream.Close
    Set objStream = Nothing
    Exit Function
    
ErrorHandler:
    Select Case Err.Number
        Case Else
           ' If there are any errors then the file was not created
            If boolLogError Then
                Call LogError("CreateUnicodeCSV", "Could not create " & strDestinationFile & " from " & strSourceFile & ": " & Err.Description)
            End If
            
            Resume ExitProcedure
    End Select

End Function

Public Function GetLongFilename(ByVal sShortName As String) As String
' From http://msgroups.net/microsoft.public.vb.general.discussion/how-do-i-display-full/139301
' 2014-10-23 JPD Created

    Dim sLongName   As String
    Dim sTemp       As String
    Dim iSlashPos   As Long
    
    sShortName = Replace(sShortName, """", vbNullString, 1)
    sShortName = sShortName & "\"
    
    iSlashPos = InStr(4, sShortName, "\")
    
    While iSlashPos
        sTemp = Dir(Left$(sShortName, iSlashPos - 1), vbNormal + vbHidden + vbSystem + vbDirectory)
        If sTemp = vbNullString Then
            GetLongFilename = vbNullString
            Exit Function
        End If
        sLongName = sLongName & "\" & sTemp
        iSlashPos = InStr(iSlashPos + 1, sShortName, "\")
    Wend
    
    GetLongFilename = Left$(sShortName, 2) & sLongName
    
End Function


